/****************************************************************************/
/*                 program helpray.c                                        */
/****************************************************************************/
/*
 *      Copyright (C) 1992,  Woods Hole Oceanographic Institution
 *    See LICENSE file for use, copying and distribution conditions
 *
 *  jim bowlin Aug 1992.
 *
 *  Contains text of detailed help message and license message.
 */


#include <stdio.h>
#include <stdlib.h>
#include "utils.h"


/****************************************************************************/
/*                        global variables                                  */
/****************************************************************************/

char *Help_text[] =
{
"",
"",
"Ray Program Inputs",
"___________________",
"",
"Ray uses several input files.  It always needs an initialization file which is",
"given as the first command line parameter.  Within the initialization file",
"three other input files may be specified; MPP input, profile input and",
"bathymetry input.",
"",
"",
"Initialization File",
"___________________",
"",
"The initialization file is always specified by the first command line",
"parameter when Ray is run.  For example,",
"",
"   Ray initialize.ray",
"",
"will run the Ray program with ``initialize.ray\'\' as the initialization file.",
"The initialization file is in ASCII.  It is always the first file read by Ray.",
"A description of its format follows below.",
"",
"The initialization file format will be familiar to people who work with the c",
"programming language.  All text inclosed between \"/*  ... */\" is a comment",
"and ignored.  There are seven groups of parameters that are specified in",
"this file. They are: input, output, source, receiver, parameters, model, and",
"save.  The parameters within each group can be specified in two different",
"ways.  One way is a single line (or statement) for each parameter.  For",
"example,",
"",
"   model integration    = rk_2;",
"   model range_depend   = grad_z;",
"",
"will set the integration routine to be ``rk_2\'\' and the range dependence to be",
"of the type ``grad_z\'\'.  More information on exactly what these mean will",
"follow below. Notice the semicolon at the end of each statement.  The second",
"method of specifying parameters within a group,",
"",
"   model {",
"      integration   = rk_2;",
"      range_depend  = grad_z:",
"      ...",
"   };",
"",
"sets these two parameters exactly like the example given previously.  This",
"method saves on redundant typing and encourages putting all of the parameters",
"within a single group together.  There are currently six types of parameters",
"that may be specified: strings, dimensioned numerical, array of dimensioned",
"numerical, dimensionless numerical, choices and flag.",
"",
"",
"String Parameters",
"_________________",
"",
"String parameters are used for specifying filenames. The name must always be",
"inside a pair of double quotes as in",
"",
"   input prof_file = \"test1.ssp\";",
"",
"",
"Dimensioned Numerical Parameters",
"________________________________",
"",
"Dimensioned numerical parameters are input as a number followed by units in",
"parentheses.  For example",
"",
"   receiver range = 1000.0 (km);",
"",
"specifies that the receiver is at a range of 1000 kilometers.  The inclusion",
"of units is not optional but a choices of units are available. Two kinds of",
"units are used, lengths and angles.  Internally lengths are all in meters.",
"The following length units are valid:",
"",
"   unit             name                  conversion to meters",
"   ____             ____                  ____________________",
"",
"   (m)              meter                      1.0e0",
"   (km)             kilometer                  1.0e3",
"   (Nmi)            nautical mile              1.852e3",
"   (ft)             foot                       3.048e-1",
"   (furlong)        furlong                    2.0117e2",
"   (parsec)         parsec                     3.804e16",
"   (cm)             centimeter                 1.0e-2",
"   (mm)             millimeter                 1.0e-3",
"   (Mm)             megameter                  1.0e6",
"   (lightyear)      lightyear                  9.46e15",
"   (angstrom)       angstrom                   1.0e-10",
"",
"Internally angles are all in radians.  The following angle units are valid:",
"",
"   unit             name                  conversion to radians",
"   ____             ____                  ____________________",
"",
"   (radians)        radians                    1.0e0",
"   (degrees)        degrees                    3.14159265358979 / 180.0",
"",
"",
"",
"Array of Dimensioned Numerical Parameters",
"_________________________________________",
"",
"An array of dimensioned numerical parameters is specified like a single",
"dimensioned parameter except the single number is replaced by a left brace,",
"a list of numbers and a closing right brace as in ",
"",
"   angles specific = {",
"      1.0, 2.0, 3.0 } (degrees);",
"   };",
"",
"Note that the list of numbers can be delimited by either whitespace (which",
"consists of: space, tab, linefeed, and carriage return) or commas.",
"",
"",
"Dimensionless Numerical Parameters",
"__________________________________",
"",
"Dimensionless numerical parameters must not have any unit specification,",
"even an empty ``()\'\' is not allowed.  For example",
"",
"   model margins = 100;",
"",
"is a valid statement.",
"",
"",
"Choice Parameters",
"__________________",
"",
"Choice parameters give the user a specific range of choices such as",
"",
"   model integration = rk_2;",
"",
"which will set the integration routine to be \"rk_2\".  Other choices for this",
"parameter are: \"rk_23\" and \"rk_4\".  If a choice parameter is misspelled or",
"otherwise inappropriate, then an error message will be generated showing where",
"the problem occurred and what the valid choices are.",
"",
"",
"Flag Parameters",
"_______________",
"",
"Flag parameters look similar to choice parameters.  A flag parameter",
"is set by mentioning its name as in:",
"",
"   save wavefront;",
"",
"which will ensure that all of the wavefront variables for this run",
"are put into the output file.",
"",
"",
"Initialization Parameters by Group",
"__________________________________",
"",
"There are seven groups of parameters that can be specified in the",
"initialization file.  They are: input, output, source, receiver, angles paths,",
"and model.  ",
"",
"",
"Input Group",
" ___________",
"",
"The input group specifies the addition files that will be read in by Ray.",
"For example",
"",
"   input {",
"      prof_file = \"test1.ssp\";",
"      bath_file = \"test1.bth\";",
"   };",
"",
"specifies that the file \"test1.ssp\" will be read in as a profile file",
"and the file \"test1.bth\" will be read in as a bathymetry file.",
"",
"Output Group",
"____________",
"",
"The output group specifies the name of the output file and what variables will",
"be written into this file.  The line",
"",
"",
"   output file = \"test1.mat\";",
"",
"tells Ray to name the output file \"test1.mat\".  The contents of this file",
"can be tailored by setting one or more of the following output flag parameters",
"",
"      output                             effect",
"   ______________          ______________________________________",
"",
"   initialization;         save scaler initialization parameters",
"   filenames;              save filenames",
"   date;                   save time and date of run",
"   sound_speeds;           save sound speed and rangestep tables",
"   bathymetry;             save bathymetry tables",
"   paths;                  save path information along each ray",
"   wavefront;              save all ray parameters at receiver range",
"   everything;             save all of the above",
"   environment_only;       don\'t trace rays",
"",
"",
"If no output flags are specified then Ray assumes everything should be",
"saved.  In order to save path information the \"paths fixed_dr\" or ",
"\"paths steps_per\" must be explicitly set.  If \"environment_only\" is ",
"selected then no rays will be traced regardless of the state of any",
"of the other flags.",
"   ",
"",
"Source Group",
"____________",
"",
"The source group specifies the depth and range of the source.  The lines",
"",
"   source {",
"      depth = 600.00 (m);",
"      range = 1000   (km);",
"   }",
"",
"will put the source 600 m below the surface at a range of 1,000 km",
"",
"",
"Receiver Group",
"______________",
"",
"The receiver group specifies the depth and range of the receiver.",
"Right now the depth of the receiver is never used by Ray.   For example:",
"",
"   receiver {",
"      depth = 1000.00  (m);",
"      range = 3000.0   (km);",
"   }",
"",
"tells Ray to put the receiver at a depth of 1000 m and a range 3000 km.",
"",
"",
"Angle Group",
"___________",
"",
"The angle group contains all of the information concerning which ",
"launch angles will be used for a ray trace.  One method of specifying",
"these angles is ",
"",
"   angles  {",
"      first =  15 (degrees);",
"      last  = -15 (degrees);",
"      number =  7;",
"   };",
"",
"which tells Ray to shoot seven rays at the angles 15, 10, 5, 0, -5, -10, and",
"-15 degrees.  Alternatively, specific angles may be selected as in",
"",
"   angles specific = { 1, 2, 3} degrees;",
"",
"It is an error to specify the launch angles in both formats.",
"",
"",
"Paths Group",
"___________",
"",
"The paths group controls how (and if) information along the path of",
"each ray is saved.  The statements",
"",
"   paths {",
"      min_range = 1000 (km);",
"      max_range = 1100 (km);",
"      fixed_dr  = 0.5 (km);",
"   };",
"",
"",
"will save path information for each ray every 1/2 km over the",
"region of 1000 to 1100 km.  If the min_range and max_range are not",
"specified then they default to the source and receiver ranges.  If",
"\"steps_per\" is used instead of \"fixed_dr\" then path information will",
"be saved every steps_per steps.  Since the step size will generally ",
"vary along each ray, the range spacing of each path will vary.  The",
"information that is stored at points along a path can be tailored by ",
"setting one or more of the following flag parameters",
"",
"",
"   paths columns                       effect",
"   __________________           _____________________________",
"   ",
"   range;                       range of the point",
"   depth;                       depth of the point",
"   time;                        time it took to get to point",
"   angle;                       angle of ray at point",
"   speed;                       sound speed at point",
"   grad;                        dc/dz at point",
"   top_bounces;                 number of top bounces",
"   bot_bounces;                 number of bottom bounces",
"   ray_number;                  which ray this point belongs to",
"   everything;                  all of the above",
"",
"If no columns are selected then they are all included.  In addition, if",
"the flag ",
"",
"   paths include bounces;",
"",
"is set then every point where a ray bounces is included in the path",
"in addition to all of the points generated by \"fixed_dr\" or \"steps_per\".",
"",
"For eigenray paths, there are 3 additional parameters to set:",
"",
"    geo_miss = 5 (m);      The allowable vertical miss distance for eigenrays",
"    diff_miss = 0 (m);     (currently unimplemented) Allows searching for",
"                           diffracted arrivals",
"    search_max = 5;        Cutoff parameter used to identify numerical",
"                           instability in the eignray search",
"",
"When searching for eigenrays, be aware that as range and complexity of",
"the sound-speed field increases, solutions may begin to exhibit chaotic",
"behavior, i.e. the range integration may start to depend critically on",
"the range steps chosen. Thus the numerical \"noise\" increases, and so",
"one must adjust geo_miss accordingly. A prime indicator of this instability",
"is that predicted ray depths at the receiver range are not a monotonic",
"function of source angle, making convergence of the search algorithm",
"unlikely. search_max gives an upper limit for the possible",
"violations of non-monotonicity; it should not be set too small because",
"there are some cases where receiver depths are near ray turning depths",
"in which the algorithm will initially seem to diverge but will eventually",
"converge.",
"",
"Also note that this numerical instability will sometimes show up as eigenrays",
"that do not come very near to the receiver. These really should be identified",
"by the program but for technical reasons this is difficult to do simply.",
"",
"Model Group",
"___________",
"",
"",
"The model group of parameters control how Ray models the ocean.  The",
"\"model integration =\" choice determines the integration routine that",
"will be used.  There are three possibilities:",
"",
"   model integration =                     effect",
"   ____________________              ________________________",
"         ",
"        rk_2;                        use 2nd order Runge Kutta",
"        rk_23;                       use 2nd - 3rd order Runge Kutta",
"        rk_4;                        use 4th order Runge Kutta",
"",
"",
"",
"The \"model range_depend =\" choice controls the way range dependence",
"is handled.",
"",
"   model range_depend =                   effect",
"   _____________________           _________________________________",
"",
"   none;                           no interpolation in range",
"   grad_z;                         linear interpolation, ignore dc/dr",
"   full;                           linear interpolation, include dc/dr",
"",
"",
"The \"model bathymetry =\" choice controls what happens when a ray touchs ",
"the bottom.",
"",
"   model bathymetry =              effect",
"   __________________              ____________________________________",
"",
"   none;                            no bathymetry",
"   soft;                            pass through bottom",
"   absorbing;                       stop at bottom",
"   reflecting;                      reflect off of bottom",
"",
"",
"The \"soft\" choice records the closest approach to the bottom ( or the",
"deepest descent into the bottom) of each ray.",
"",
"   Within the model group, the range_step subgroup controls the size of the",
"range steps taken by the integration routines.  Ray provides a facility for a",
"depth dependent step size.  The shape of this function is controlled by the",
"\"prof_smoothing\" widths.  But the overall size of the steps can be controlled",
"as in ",
"",
"   parameter range_step multiplier = 0.5;",
"",
"which halves all of the range steps.  The maximum and minimum range step can",
"also be controlled.  This process occurs after the multiplier has been",
"applied so that the units used for the minimum and maximum are not scaled by",
"the multiplier.  The lines",
"",
"  model range_step {",
"      max = 10 (m);",
"      min = 10 (m);",
"  }",
"",
"will set the range step size to 10 m for all depths regardless of the value of",
"the multiplier.  The maximum and minimum can also be set to different values.",
"If the maximum is less than the minimum then the step size is set to the",
"minimum.",
"",
"There are two parameters that can be adjusted in the debias",
"subgroup.  These parameters control how close Ray stays to the input sound",
"speed profiles.  They are both dimensionless numerical parameters and are set",
"as in the following example",
"",
"  model debias {",
"     factor = 1;",
"     iteration = 10;",
"  }",
"",
"",
"There are six numerical parameters in the model group.  These parameters",
"and their default values are",
"",
"   model {",
"      prof_smoothing = { ",
"           5,   5,   5,   5,   5,  20,  12,  13,",
"          12,  25,  25,  50,  50,  50,  50,  50,  ",
"          50,  50,  50,  50,  50,  50,  50,  50, ",
"         125, 125, 250, 250, 250, 250, 250, 250  }  (m);",
"        ",
"      bath_smoothing =     10 (km);",
"      bottom_depth   =   6000 (km);",
"      earth_radius = 6378.137 (km);",
"      z_tolerance  =     1e-6 (m) ;",
"      margins      =      100     ;        ",
"   };",
"",
"The \"prof_smoothing\" array contains the widths used with for smoothing the",
"sound speed profiles.  The widths are used in order so that the first width",
"is used for smoothing the first point in each profile, the second width is",
"used for the second point, and so on.  The default smoothing widths were",
"chosen to work well with profiles based on the levitus depth spacing.",
"The \"bath_smoothing\" is used for smoothing the bathymetry in a similar",
"manner.  If it is set to 0 then no bathymetry smoothing is performed and",
"Ray creates a bottom made up of piecewise linear segments connecting the",
"input bathymetry points.  The \"bottom_depth\" parameter sets the absolute",
"maximum depth when no bathymetry is input.  If any bathymetry point lies",
"below \"bottom_depth\" then \"bottom_depth\" is set to the deepest bathymetry",
"point.  The \"earth_radius\" parameter is used for the spherical earth ",
"correction.  If it is set to 0 then no spherical earth correction is used.",
"The \"z_tolerance\" parameter controls how close a ray need get to boundary",
"before it bounces off.  The \"margins\" parameter sets how much extra depth",
"is provided over the top and under the bottom.  The default values of these",
"parameters should work fine in most situations.  If there are set incorrectly",
"then spurious results may be obtained.",
"",
"",
"Command Line Substitution",
"_________________________",
"",
"Often the situation arises when several ray traces need to be performed that",
"differ in only one or two parameters.  To facilitate work on such problems Ray",
"has the ability to do command line substitution in the initialization file.",
"This allows repeated use of the same initialization file for a number of ray",
"traces.  Everywhere in the initialization file where a ``$1$\'\' occurs, the",
"second command line parameter is substituted for the ``$1$.\'\' Everywhere that",
"a ``$2$\'\' is encountered, the third command line parameter is substituted for",
"every ``$2$.\'\'  Ray allows up to 30 replaceable parameters to be specified",
"although some operating systems may limit the total number of command line",
"parameters to less than 30.  For example suppose a file called ``init.ray\'\'",
"contains the lines",
"",
"   input prof_file = \"$1$.prf\";",
"   input bath_file = \"$1$.bth\";",
"   model integration = $2$;",
"",
"and then suppose that Ray is invoked by",
"",
"   Ray init.ray  test1 grad_z",
"",
"then this will have the same effect as if the initialization file contained",
"",
"   input prof_file = \"test1.ssp\";",
"   input bath_file = \"test1.bth\";",
"   model integration = grad_z;",
"",
"Note that although replaceable parameters are allowed to split a string (as in",
"\"$1$.prf\" above) it is not valid to try to split up a word that is not in",
"quotes with a replaceable parameter.  For example, the line",
"",
"   model integration rk_$1$;",
"",
"is not valid and will generate several error messages.",
"",
"Although Ray allows users to adjust a wide variety of input parameters, most",
"parameters do not need to be specified.  There are some parameters that do ",
"need to get set for every run.  below is a listing of the minimum amount of",
"information needed in an initialization file for a ray trace to be performed.",
"",
"   input prof_file = \"test1.ssp\";     /* sound speed profile input file */",
"   output mat_file = \"test1.mat\";     /* MatLab output  file            */",
"   source    depth = 1000 (m);        /* depth of source                */  ",
"   receiver  range = 2000 (km);       /* range of receiver              */",
"   angles    first =   15 (degrees);  /* extremes of the fan of rays    */",
"   angles     last =  -15 (degrees);  /* extremes of the fan of rays    */",
"   angles   number =  500;            /* number of rays to shoot        */ ",
"",
"If a bathymetric run is desired then just add the following line to the above",
"",
"   input bath_file = \"test1.bth\";     /* bathymetry input file          */",
"",
"and Ray will automatically set the bathymetry to reflecting.  If path ",
"information is desired then add the following line to the initialization",
"file",
"",
"   paths fixed_dr = 1000 (m);        /* range step of path information  */",
"",
"which will cause Ray to save information along the entire path of each ray",
"from the source to the receiver every 1000 m.",
"",
"",
"Ray program outputs",
"___________________",
"",
"The output from Ray is a single MatLab file.  The name of this file is",
"specified in the initialization file on a line of the form",
"",
"   output file = \"fname.mat\";",
"",
"This file is written in binary double precision MatLab format and can be read",
"be directly by the MatLab program on a variety of platforms.  It contains a",
"collection of MatLab variables that can totally specify the inputs and outputs",
"of the Ray program.  The variables are divided up into seven groups.  The user",
"has control over which group or groups of variables get saved by specifying",
"\"output\" flags in the initialization file.  ",
"",
"The names of the seven output groups are given in the table below along with a",
"brief description of what each group contains.",
"",
"output group            contents",
"______________        _________________________________________________",
"",
"initialization;         all scaler parameters in initialization file",
"filenames;              all filenames in initialization file",
"date;                   date and time of ray trace",
"sound_speeds;           sound speed profiles tables",
"bathymetry;             bathymetry table",
"paths;                  path of each ray",
"wavefront;              parameters of each ray at receiver range",
"",
"",
"Initialization Group",
"____________________",
"",
"The initialization group contains just one variable named ``inits.\'\'",
"Its has a size of 1 by 17 and contains all of the information (except",
"filenames) contained in the initialization file. Its contents are",
"detailed below.",
"",
"   inits(1)  = ray version number",
"   inits(2)  = source range (m)",
"   inits(3)  = source depth (m)",
"   inits(4)  = receiver range (m)",
"   inits(5)  = receiver depth (m)",
"   inits(6)  = first angle to be launched at source (radians)",
"   inits(7)  = last angle to be launched at source  (radians)",
"   inits(8)  = number of rays to be launched (integer)",
"   inits(9)  = starting range for saving paths",
"   inits(10) = ending range for saving paths",
"   inits(11) = fixed path step size",
"   inits(12) = Runge Kutta steps per path step",
"   inits(13) = range dependence choice ( 0-3 )",
"          0 => dc/dz range dependence     (default)",
"          1 => no range dependence        \"none\"",
"          2 => dc/dz range dependence     \"grad_z\"",
"          3 => full range dependence      \"full\"",
"",
"   inits(14) = integration routine choice ( 0-3 )",
"          0 => 4th order runge kutta      (default)",
"          1 => 2nd order runge kutta      \"rk_2\"",
"          2 => 2nd-3rd order runge kutta  \"rk_23\"",
"          3 => 4th order runge kutta      \"rk_4\"",
"",
"   inits(15) = bathymetry choice (0-4)",
"          0 => none (or reflecting)       (default) ",
"          1 => no bathymetry (flat)       \"none\"",
"          2 => soft bottom                \"soft\"",
"          3 => absorbing bottom           \"absorbing\"",
"          4 => reflecting bottom          \"reflecting\"",
"",
"   inits(16) = step size multiplier",
"   inits(17) = maximum step size (m)",
"   inits(18) = minimum step size (m)",
"   inits(19) = debias factor ",
"   inits(20) = debias iteration",
"   inits(21) = bathymetry smoothing width (m)",
"   inits(22) = bottom depth (m)",
"   inits(23) = radius of earth (m)",
"   inits(24) = z tolerance (m)",
"   inits(25) = margins",
"",
"Note: Depth scale is zero at the surface and positive downward so",
"      that all depths should be greater than or equal to zero.",
"Note: Angles are defined so positive angles indicate rays moving upward",
"      toward the surface.",
"",
"",
"Filenames Group",
"_______________",
"",
"The filenames group contains the names of all of the filenames",
"specified in the initialization file plus the name of the intialization",
"file.  Their sizes depend on the string length of the filenames. They",
"are all MatLab text variables.",
"",
"   variable name              contents",
"   _____________            ____________________________",
"   bath_file                  bathymetry input filename",
"   init_file                  initialization input filename",
"   out_file                   output filename",
"   prof_file                  profile input filename",
"",
"",
"Date Group",
"__________",
"",
"The date group contains two variables.  One is a text variable named",
"\"run_date\" which contains the date and time that the ray trace started.",
"The second variable is named \"runtime\" and contains the number of seconds",
"that elapsed while Ray was performing the ray trace.",
"",
"",
"Sound Speed Group",
"_________________",
"",
"These variables contain the tables that Ray uses internally for modeling the",
"speed of sound and the range step size.  The sizes of many of these variables",
"will vary depending upon the data in the input files.  Some useful numbers",
"are:",
"",
"Nss          The number of sound speed profiles.",
"",
"Nbat         The number of bathymetry sections ( which will be",
"             approximately twice the number of bathymetry points).",
"",
"Nseg         The number of segments used for generating sound speed",
"             profiles (usually twice to three times the maximum number",
"             of points input in any sound speed profile.",
"",
"",
"name (size)                      description",
"_________    _______________________________________________________",
"",
" sspr         The ranges (in meters) of the Nss sound speed profiles.",
"(1 by Nss)",
"",
"",
" idex        A table of indices for use with ctab### rec_c and rtab.",
"(Nz by 1)    For every meter of depth from 0 to Nz, idex(depth+1)",
"             will tell you which parameterization of the ocean to",
"             use for that depth.",
"             In order to calculate a sound speed (or a step size) the",
"             depth in meters is used as an index into this array. For",
"             example if you want to find a sound speed at a depth of",
"             z meters then you look at ii = idex([z]+1) where [z] is the",
"             integer part of z.  The parameters in  ctab###(ii + 1,:) are used",
"             to compute the sound speed.",
"",
" ctab###     Tables used to construct sound speed profiles.  The symbol",
"(Nseg by 4)  ### will vary from 001 to the Nss where Nss is the number",
"             of sound speed profiles. They are ordered by increasing range",
"             from the source.",
"             Column 1 is the depth we expand about:     z0.",
"             Column 2 is the sound speed at z0:         c0.",
"             Column 3 is dc/dz at z0:                   c1.",
"             Column 4 is one half of d^2c /dz^2 at z0:  c2.",
"             Sound speeds c(z) are calculated as follows:",
"                ii = idex([z] + 1) + 1; ",
"                z0 = ctab(ii,1);",
"                c0 = ctab(ii,2);",
"                c1 = ctab(ii,3);",
"                c2 = ctab(ii,4);",
"                dz = z = z0;",
"                c(z) = c0 + dz * ( c1 + dz * c2);",
"",
" rec_c       Table used to calculate the sound speed at the receiver range.",
"(Nseg by 4)  This is useful since the receiver range may not coincide with",
"             any of the ranges of input sound speed profiles.  The format",
"             is identical to that used in ctab### above.",
"",
" rtab        Table used to calculate the range step as a function of depth.",
"(Nseg by 5)  Its use is similar to btab and ctab described above.  The fifth ",
"             column is used by Ray for storing auxiliary width information ",
"             and is not needed for computing the range step size.",
"",
"",
"",
"Bathymetry Group",
"",
" btab        The table used to determine the piecewise parabolic bottom.",
"(Nbat by 5)  Column 1 is the beginning range of each piece:    r0.",
"             Column 2 is the depth at the beginning range:     b0.",
"             Column 3 is db/dr at r0:                          b1.",
"             Column 4 is one half of d^2b /dr^2 at r0:         b2.",
"             Column 5 is the minimum depth of this section:   minz.",
"             The bottom depth b(r) is calculated as follows:",
"             Find the entry in btab such that",
"                btab(i-1,1)  <=  r  <=  btab(i,1)",
"             then let",
"                r0 = btab(i,1);",
"                b0 = btab(i,2);",
"                b1 = btab(i,3);",
"                b2 = btab(i,4);",
"                dr = r - r0;",
"             then",
"                b(r) = b0 + b1 * dr + b2 * dr * dr;",
"",
"             Note that the number of rows in this table will be",
"             approximately twice as the number of input bathymetry points.",
"             If there is no bathymetry then this variable will not exist.",
"",
"",
"Wavefront Group",
"_________",
"",
"The wavefront group contains one variable named \"wf\" which stands for",
"\"wavefront.\"  Its size is Nr by 20 where Nr is the number of rays",
"that were shot.  ",
"",
"   wf(1)  = angle at the source   (radians)",
"   wf(2)  = depth at the source   (radians)",
"   wf(3)  = range of source       (m)",
"   wf(4)  = angle at the receiver (radians)",
"   wf(5)  = depth at the receiver (radians)",
"   wf(6)  = range of receiver     (m)",
"   wf(7)  = travel time           (s)",
"   wf(8)  = number of top turns",
"   wf(9)  = number of bottom turns",
"   wf(10) = number of top bounces",
"   wf(11) = number of bottom bounces",
"   wf(12) = total number of turns",
"   wf(13) = total number of bounces",
"   wf(14) = upper turning depth at receiver (m)",
"   wf(15) = lower turning depth at receiver (m)",
"   wf(16) = sin(theta)/c",
"   wf(17) = sound speed at receiver range",
"   wf(18) = sound speed at source (m/s)",
"   wf(19) = arc-length (m)",
"   wf(20) = arc-length/c^2 (s^2/m)",
"",
"Note: ** indicates variables that are not fully implemented.",
"",
"The upper turning depth at receiver and lower turning depth at receiver are",
"the upper and lower turning depths that would occur if the ray were continued",
"past the receiver range with a range independent sound speed profile identical",
"to the profile at the receiver.",
"",
"",
"Paths Group",
"___________",
"",
"The paths group contains the one variable \"paths\" which contains all of the",
"path information for all of the rays that were traced.  The number of columns",
"in \"paths\" depends on which columns were selected in the initialization file.",
"If all of the columns were selected then \"paths\" will have nine columns ",
"containing the following information",
"",
"   paths(1) = range (m)",
"   paths(2) = depth (m)",
"   paths(3) = time  (s)",
"   paths(4) = angle (radians)",
"   paths(5) = sound speed (m/s)",
"   paths(6) = dc/dz (1/s)",
"   paths(7) = arc-length (m)",
"   paths(8) = arc-length/c^2  (s^2/m)",
"   paths(9) = sin(theta)/c",
"   paths(10) = number of top bounces",
"   paths(11) = number of bottom bounces",
"   paths(12) = the ray number",
"      ",
"If a subset of the possible columns are selected then the number of columns",
"will be reduced but the order of the remaining columns will always be as",
"shown above.  The first ray shot is number one, the second is number two,",
"and so on.",
""
};


/*
 *    What follows is the text of the license message.
 */

char *License_text[] =
{
"",
"",
"\t\t      Copyright Notice",
"\t     Woods Hole Oceanographic Institution",
"\t      Woods Hole, Massachusetts   02543",
"\t\t\t July 1992",
"",
"   Copyright (C) 1992-94 Woods Hole Oceanographic Institution",
"",
" Permission to use, copy, modify, and distribute this software and its",
" documentation for any purpose without fee is hereby granted, provided:",
"",
"   - that the above copyright notice appear in all copies.",
"",
"   - that both the copyright notice and this permission notice appear",
"     in supporting documentation.",
"",
"   - that copyright notices displayed during software operation remain",
"     unaltered.",
"",
"   - that the name Woods Hole Oceanographic Institution not be used in",
"     advertising or publicity pertaining to distribution of the software",
"     without specific prior written permission.",
"",
"   - that this software not be a part of any for-profit product without",
"     prior written permission or license from Woods Hole Oceanographic",
"     Institution.",
"",
"\t\t      Disclaimer Notice",
"",
" Woods Hole Oceanographic Institution makes no representations about the",
" suitability of this software for any purpose.  It is provided \"as is\"",
" without express or implied warranty.  It is provided without obligation",
" of support on the part of Woods Hole Oceanographic Institution to",
" assist in its use, correction, modification, or enhancement.",
"",
""
};

char *Whatsnew_text[] =
{
"",
"                 WHATS NEW FOR RAY VERSION 1.10",
"\t\t _______________________________",
"",
"0)   MPP",
"     ___",
"",
"     Ray no longer supports the MPP input file format.",
"",
"",
"1)   New *.spp sound speed profile file format.",
"     _________________________________________",
"",
"",
"     The new format is described below:",
"",
"   neg        r_1\t r_2  \t r_3 ...   r_n",
"   z_1       c_11 \tc_12 \tc_13 ...  c_1n",
"   z_2       c_21 \tc_22 \tc_23 ...  c_2n",
"\t     .  .  .",
"   z_m       c_m1 \tc_m2 \tc_m3 ...  c_mn",
"",
"   Where:",
"    neg\tis any negative number.",
"    r_j\tis the range of the j\'th profile (km).",
"    z_i\tis the depth of the i\'th point (m).",
"   c_ij is the sound speed at range r_j and depth z_i (m/s).",
"",
"   This block structure can be repeated indefinitely.",
"",
"2)  Debias changes.",
"    _______________",
"",
"   A)  debias factor has been eliminated.  It is now always set to 1.0.",
"",
"   B)  In addition to setting a maximum number of iterations the",
"       user can also specify the sound speed tolerance ( m_per_s).",
"",
"   C)  Exact debiasing is now available.  Please do not use it blindly.",
"       It is known to be unstable.  Always check the output profiles",
"       before doing a run.  The smaller the prof_smoothing widths the",
"       more stable the debiasing seems to be.",
"",
"   D)  \"prof_smoothing max_auto;\" is now available which will automatically",
"       choose smoothing widths (the maximum possible) for an input depth",
"       grid.  Note that these may not be optimal and were provided to",
"       let users get up and running quickly.  REMEMBER: always check",
"       the profiles.",
"",
"3)     Wide angle rays",
"       _______________",
"",
"       There have been quite a number of \"start below bottom\" errors.",
"       This should now be fixed.  Ray should now work with very steep",
"       ray angles ( +/- 99.5 (degrees) ).  The basic change was to ",
"       scale the step size by the cosine of the angle of the ray.",
"       The \"start below bottom\"  problem was fixed by changing",
"",
"                      dz approx  2 * sin(theta) * dr",
"                                to",
"                      dz approx  2 * tan(theta) * dr",
" ",
"",
"4)     Max_bounces",
"       ___________",
"",
"       To go along with these steep ray angles I have added a max_bounce",
"       parameter to abort rays after they bounce max_bounce times.",
"       Setting \"max_bounce = 0;\" turns off this feature.",
"",
"5)    Output formats",
"      ______________",
"",
"      A) Ascii output is now available.  Only",
"",
"\t       initialization parameters",
"\t       filenames",
"\t       paths",
"\t       wavefronts",
"",
"\t are currently put into the ascii file.  For other outputs",
"\t please buy MatLab.  It\'s a great program.",
"",
"      B) The paths variable name in the output has been changed to \"pinfo\".",
"",
"      C) Some more information on profile smoothing and widths is put",
"\t into the output file when \"output soundspeeds;\" is selected.",
"",
"      D) There are now more columns available to save along each path.",
"         Some of the unused columns in the \"wf\" variable have changed.",
"         Look at the MatLab variable \"wf_help\" for details.  Or look",
"         at the rinfos structure in the file <ray.h>.",
"",
"6)  The Integration routines have been drastically modifed.  Only",
"    4th order Runge-Kutta is supported.  It should run faster now.",
"    Daring souls can fiddle with declaring some of the critical",
"    loop local variable \"register\" instead of \"static\" and check",
"    speed differences.",
"",
"7)  Ray now uses default extensions.  They are",
"",
"       initfile:   *.ray",
"      prof_file:   *.ssp",
"      bath_file:   *.bth",
"       mat_file:   *.mat",
"     ascii_file:   *.asc ",
"",
"8)  There is now a log file named $0$.log which records some diagnostics",
"    for the ray run.",
"",
"9)  Inner loop error and exception handling has been improved.",
"",
"8)  There is no longer a choice of bathymetry types.  If you do",
"    not specify a \"bath_file\" then a flat bottom is used.  If",
"    you do specify a \"bath_file\" then a reflecting bottom is assumed.",
"    For an absorbing bottom set \"model max_angle = -1.0;\" or any",
"    other negative number.",
"",
"10) Eigenray searching has been added",
""
};


/****************************************************************************/
/*                 end of global variables                                  */
/****************************************************************************/

void do_help(void)
/*
 *   Print the help message and then exit from program.
 */
{
int i;
int max_i = Num_els(Help_text);

   for (i = 0; i < max_i ; i++) mputs(Help_text[i]);
   exit(0);
}

void do_license(void)
/*
 *  Print the license message and then exit from program.
 */
{
int i;
int max_i = Num_els(License_text);

   for ( i = 0; i < max_i; i++) mputs(License_text[i]);
   exit(0);
}

void do_whatsnew(void)
/*
 *  Prints whats new and then exits
 */
{
int i;
int max_i = Num_els(Whatsnew_text);

   for ( i = 0; i < max_i; i++) mputs(Whatsnew_text[i]);
   exit(0);
}

/****************************************************************************/
/*                 end of helpray.c                                         */
/****************************************************************************/

