/****************************************************************************/
/********************* header  init.h ***************************************/
/****************************************************************************/
/*
 *    Copyright (C) 1993,  Woods Hole Oceanographic Institution
 *    See LICENSE file for use, copying and distribution conditions
 *
 *    Jim Bowlin jbowlin@whoi.edu        May 10, 1993
 */

/*/  Typedefs:
*/

typedef struct intabs intabs;
typedef long (*printits)(char *s,FILE *file);

/*/  Structures:
*/

typedef enum {
   None   = 0,
   Struct,       /* points to another intab structure */
   Double,       /* points to a double                */
   Dub_array,    /* linked list of doubles (dubs)     */
   Int,          /* points to an int                  */
   Enum,         /* a choice in another intab         */
   Str,          /* a string                          */
   Flag          /* a flag                            */
} types;

struct intabs {
   char     *name;           /* name of this intab                 */
   types    type;            /* type of intab (see types above)    */
   intabs   *list;           /* see    *** comment below ***       */
   int      num;             /* number of elements in *list        */
   int      def_unit;        /* offset of default unit in *list    */
   int      set_flag;        /* number of times this intab was set */
   void     *where;          /* where to put value                 */
};

/*
 * *** comment below***  *list will point to another array of intabs,
 * the "meaning" of this array will depend on the type of intab.
 *
 *    type        meaning of *list
 *    ____        _______________________________________
 *
 *    Double      the units to be used.
 *    Struct      the intab to be recursed into.
 *    Enum        the choices this available to this intab.
 */

/*/  Functions:
*/

void read_main_tab(iofiles *iof,intabs *intab);
/*
 * Tries to read in variables specified in intab from the FILE iof->file.
 * Sends error messages to stdout ( and iof->log_file ) when any unrecognized
 * text is encountered.
 */

void print_main_tab(printits printit,FILE *file,intabs *intab);
/*
 *  Prints a human readable and editable initialization file.
 *  Using the names and variables specified in intab.
 *  This file can then be read in via read_main_tab()
 *  in order to initialize any or all of the variables
 *  listed in intab. Returns the number of bytes written.
 */

/****************************************************************************/
/********************* header  init.h ***************************************/
/****************************************************************************/
