/**************************************************************************/
/**************************** initvar.h ***********************************/
/**************************************************************************/

/*
 *    Copyright (C) 1993,  Woods Hole Oceanographic Institution
 *    See LICENSE file for use, copying and distribution conditions
 *
 *    Jim Bowlin jbowlin@whoi.edu        May 10, 1993
 */

/*/  Definitions:
*/

/*
 *   The following  list of macros are used for filling in an intabs
 *   structure.  See the intabs declarations below for examples of their use.
 */

#define STRUCT(lst)        Struct,    lst,      Num_els(lst),      0, 0, NULL
#define METERS(pnt)        Double,    Meters,   Num_els(Meters),   0, 0, &pnt
#define VELOCITY(pnt)      Double,    Velocity, Num_els(Velocity), 0, 0, &pnt
#define SECONDS(pnt)       Double,    Seconds,  Num_els(Seconds),  0, 0, &pnt
#define HERTZ(pnt)         Double,    Hertz,    Num_els(Hertz),    0, 0, &pnt
#define KMETERS(pnt)       Double,    Meters,   Num_els(Meters),   1, 0, &pnt
#define METER_ARRAY(pnt)   Dub_array, Meters,   Num_els(Meters),   0, 0, &pnt
#define DEGREES(pnt)       Double,    Radians,  Num_els(Radians),  1, 0, &pnt
#define DEGREE_ARRAY(pnt)  Dub_array, Radians,  Num_els(Radians),  1, 0, &pnt
#define ENUM(lst,pnt)      Enum,      lst,      Num_els(lst),      0, 0, &pnt
#define INT(pnt)           Int,       NULL,     0,                 0, 0, &pnt
#define UDOUBLE(pnt)       Double,    NULL,     0,                 0, 0, &pnt
#define STR(pnt)           Str,       NULL,     0,                 0, 0, &pnt
#define NONE               None,      NULL,     0,                 0, 0, NULL
#define FLAG(pnt)          Flag,      NULL,     0,                 0, 0, &pnt

/****************************************************************************/
/*                 unit conversion constants                                */
/****************************************************************************/

/*/  Global Variables:
*/

double m_per_m         = 1.0;
double m_per_km        = 1.0e3;
double m_per_mm        = 1.0e-3;
double m_per_cm        = 1.0e-2;
double m_per_Mm        = 1.0e6;
double m_per_Nmi       = 1852.0;
double m_per_ft        = 0.3048;
double m_per_furlong   = 201.17;
double m_per_parsec    = 3.084e16;
double m_per_lightyear = 9.46e15;
double m_per_angstrom  = 1.0e-10;

double rad_per_rad = 1.0;
double rad_per_deg = Pi / 180.0;

double mps_per_mps  = 1.0;
double mps_per_mph  = 0.44704;
double mps_per_ftps = 0.3048;
double mps_per_knot = 0.5155;
double speed_of_light = 2.997925e8;

double s_per_s      = 1.0;
double s_per_minute = 60.0;
double s_per_hour   = 60.0 * 60.0;
double s_per_day    = 60.0 * 60.0 * 24;
double s_per_year   = 60.0 * 60.0 * 24.0 * 365.0;
double s_per_ms     = 1e-3;
double s_per_us     = 1e-6;
double s_per_ns     = 1e-9;

double hz_per_hz   = 1.0;
double hz_per_khz  = 1e3;
double hz_per_mhz  = 1e6;
double hz_per_ghz  = 1e9;

double dB_per_dB   = 1.0;

intabs Meters[] ={

   "m",         UDOUBLE(m_per_m),
   "km",        UDOUBLE(m_per_km),
   "Nmi",       UDOUBLE(m_per_Nmi),
   "ft",        UDOUBLE(m_per_ft),
   "furlong",   UDOUBLE(m_per_furlong),
   "parsec",    UDOUBLE(m_per_parsec),
   "cm",        UDOUBLE(m_per_cm),
   "mm",        UDOUBLE(m_per_mm),
   "Mm",        UDOUBLE(m_per_Mm),
/***
   "lightyear", UDOUBLE(m_per_lightyear),
   "angstrom",  UDOUBLE(m_per_angstrom)
***/
};

intabs Radians[] = {

   "radians",  UDOUBLE(rad_per_rad),
   "degrees",  UDOUBLE(rad_per_deg)
};

intabs Attenuation[] = {

   "dB",  UDOUBLE(dB_per_dB),
};

intabs Velocity[] = {

   "m_per_s",  UDOUBLE(mps_per_mps),
   "mph",      UDOUBLE(mps_per_mph),
   "ft_per_s", UDOUBLE(mps_per_ftps),
   "knots",    UDOUBLE(mps_per_knot),
   "c",        UDOUBLE(speed_of_light)
};

intabs Seconds[] = {
   "s",      UDOUBLE( s_per_s      ),
   "m",      UDOUBLE( s_per_minute ),
   "h",      UDOUBLE( s_per_hour   ),
   "day",    UDOUBLE( s_per_day    ),
   "year",   UDOUBLE( s_per_year   ),
   "ms",     UDOUBLE( s_per_ms     ),
   "us",     UDOUBLE( s_per_us     ),
   "ns",     UDOUBLE( s_per_ns     )
};       
intabs Hertz[] = {
   "Hz",      UDOUBLE( hz_per_hz  ),
   "kHz",     UDOUBLE( hz_per_khz ),
   "mHz",     UDOUBLE( hz_per_mhz ),
   "gHz",     UDOUBLE( hz_per_ghz ),
};       
   
/**************************************************************************/
/************************ end of initvar.h ********************************/
/**************************************************************************/
