/****************************************************************************/
/*                 program outray.c                                         */
/****************************************************************************/
/*
 *      Copyright (C) 1992,  Woods Hole Oceanographic Institution
 *    See LICENSE file for use, copying and distribution conditions
 *
 *
 *  jim bowlin feb 1992.
 *
 *  matlab output routines for ray.c
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "utils.h"
#include "ray.h"
#include "outray.h"

/****************************************************************************/
/*                    Global Variables                                      */
/****************************************************************************/

char *Wf_help[] = {
   "s_angle    = %15s(:,%2d);",
   "s_depth    = %15s(:,%2d);",
   "s_range    = %15s(:,%2d);",
   "r_angle    = %15s(:,%2d);",
   "r_depth    = %15s(:,%2d);",
   "r_range    = %15s(:,%2d);",
   "arr_time   = %15s(:,%2d);",
   "top_turns  = %15s(:,%2d);",
   "bot_turns  = %15s(:,%2d);",
   "top_bonks  = %15s(:,%2d);",
   "bot_bonks  = %15s(:,%2d);",
   "tot_turns  = %15s(:,%2d);",
   "tot_bonks  = %15s(:,%2d);",
   "upper_tp   = %15s(:,%2d);",
   "lower_tp   = %15s(:,%2d);",
   "p_z        = %15s(:,%2d);",
   "c_at_r     = %15s(:,%2d);",
   "c_at_s     = %15s(:,%2d);",
   "length     = %15s(:,%2d);",
   "length_c2  = %15s(:,%2d);",
   "geo_intens = %15s(:,%2d);",
   "bot_loss   = %15s(:,%2d);",
   "tot_intens = %15s(:,%2d);",
   "tot_amp    = %15s(:,%2d);",
};

char *Path_help[] = {
"p_range     = %15s(:,%2d);",
"p_depth     = %15s(:,%2d);",
"p_time      = %15s(:,%2d);",
"p_angle     = %15s(:,%2d);",
"p_speed     = %15s(:,%2d);",
"p_grad      = %15s(:,%2d);",
"p_length    = %15s(:,%2d);",
"p_length_c2 = %15s(:,%2d);",
"p_P_z       = %15s(:,%2d);",
"p_top_bonks = %15s(:,%2d);",
"p_bot_bonks = %15s(:,%2d);",
"p_tot_bonks = %15s(:,%2d);",
"upper_tp    = %15s(:,%2d);",
"lower_tp    = %15s(:,%2d);",
"bottom_loss = %15s(:,%2d);",
"p_number    = %15s(:,%2d);"};


/****************************************************************************/
/*                 end of Global Variables                                  */
/****************************************************************************/

/****************************************************************************/
/*                       OUTPUT ROUTINES                                    */
/****************************************************************************/


long mat_type(long swaprc,long dtype,long text)
{
/*** MOPT ***/
   return( 1000 * Mat_mach_num + 100 * swaprc + 10 * dtype  + text);
}

void start_mat(FILE *file,char *name,LONG rows,LONG cols,long iflag,long type)
{
struct {
   LONG type;
   LONG rows;
   LONG cols;
   LONG iflag;
   LONG namlen;
} header;

   header.type = type;
   header.rows = rows;
   header.cols = cols;
   header.iflag = iflag;
   header.namlen = strlen(name) + 1;
   fwrite(&header,sizeof(header),1,file);
   fwrite(name,1,header.namlen,file);
}

void str_2_mat(FILE *file,char *s)
{
int i;
double dub;
int n = strlen(s);

  for(i=0; i < n; i++) {
      dub = s[i];
      fwrite(&dub,sizeof(double),1,file);
   }
}


void save_mat_str(FILE *file,char *name,char *str)
{
int cols,rows = 1, iflag=0, type = mat_type(0,0,1);

   if (str == NULL) return;
   if (file == NULL) return;
   cols = strlen(str);
   start_mat(file,name,rows,cols,iflag,type);
   str_2_mat(file,str);
}

void save_string(FILE *mat_file,FILE *asc_file,char *name,char *str)
{
   if (str == NULL) return;
   if (mat_file != NULL) save_mat_str(mat_file,name,str);
   if (asc_file != NULL) fprintf(asc_file,"%s = %s\n",name,str);
}


void save_wf_help(FILE *file,char *name)
{
int cols,rows,i, iflag = 0, type = mat_type(1,0,1);
char s[100];
char vname[80] = "help_";

/*** get length of one string in array into cols ***/

   sprintf(s,Wf_help[0],name,0);
   cols = strlen(s);

/*** get number of strings in array into rows ***/

   rows = Num_els(Wf_help);
   strcat(vname,name);
   start_mat(file,vname,rows,cols,iflag,type);

   for(i=0; i< rows; i++) {
      sprintf(s,Wf_help[i],name,i+1);
      str_2_mat(file,s);
   }
}

void save_pinfo_help(FILE *file,char *name,pinfos *pinfo)
{
int cols,i;
int iflag = 0;
int type = mat_type(1,0,1);
int cnt = 0;
char s[100];

/*** get length of one string in array into cols ***/

   sprintf(s,Path_help[0],name,0);
   cols = strlen(s);
   strcpy(s,"help_");
   strcat(s,name);

   start_mat(file,s,pinfo->num_cols,cols,iflag,type);

   for(i=0; i < Num_els(Path_help); i++) {
      if (pinfo->saveflg[i] != 1) continue;
      sprintf(s,Path_help[i],name,++cnt);
      str_2_mat(file,s);
   }
}

void start_swapmat(FILE *file,char *name,LONG rows,LONG cols)
{
int type = mat_type(1,0,0);

   start_mat(file,name,rows,cols,0,type);
}

void save_index(FILE *file,int index[],int nz)
{
double d;
int    i;

   if (file == NULL) return;
   start_mat(file,Param.vname.idex,nz,1,0,mat_type(0,0,0));
   for(i= 0; i<nz; i++) {
      d = index[i+index[0]];
      fwrite(&d,sizeof(double),1,file);
   }
}

void save_table(FILE *file,c_tabs tab[],int ntab,char *name)
{
int tabsize = sizeof(tab[0]);

   if (file == NULL) return;
   start_swapmat(file,name,ntab,tabsize / sizeof(double));
   fwrite(tab,tabsize,ntab,file);
}



void save_tabtab(FILE *file,tabtabs *tabtab,int ns,char *name)
{
tabtabs *tt;
int   i;
long rows,cols;
c_tabs *ctab;

   if (file == NULL) return;
   for(i=0, tt = tabtab; tt != NULL;tt=tt->next) i++;
   cols = 4 * i;
   rows = ns;
   start_mat(file,name,rows,cols,0,mat_type(0,0,0));
   
   for(tt = tabtab; tt != NULL;tt=tt->next) {
      for(i=0,ctab = tt->tab1; i< rows; i++,ctab++)
         fwrite(&(ctab->z0),sizeof(double),1,file);      
      for(i=0,ctab = tt->tab1; i< rows; i++,ctab++)
         fwrite(&(ctab->c0),sizeof(double),1,file);
      for(i=0,ctab = tt->tab1; i< rows; i++,ctab++)
         fwrite(&(ctab->c1),sizeof(double),1,file);
      for(i=0,ctab = tt->tab1; i< rows; i++,ctab++)
         fwrite(&(ctab->c2),sizeof(double),1,file);
   }
}

void save_rstep(FILE *file,rstep_tabs *rstep,int n)
{
int size = sizeof(rstep[0]);

   if (file == NULL) return;
   start_swapmat(file,Param.vname.rtab,n,size / sizeof(double));
   fwrite(rstep,size,n,file);
}

void save_sspr(FILE *file,tabtabs *tabtab,char *name)
{
tabtabs *tt;
int   i = 0;
double dum;

   if (file == NULL) return;
   for(tt = tabtab; tt != NULL;tt=tt->next) i++;
   start_mat(file,name,1,i,0,mat_type(0,0,0));
   for(tt = tabtab; tt != NULL;tt=tt->next) {
      dum = Param.sign_dr * tt->r1;
      fwrite(&dum,sizeof(double),1,file);
   }
}

void save_mat_rinfo(FILE *file,rinfos *first,char *name)
{
rinfos *rinfo;
int sizeb,sized;
int n = 0;

   for(rinfo = first; rinfo != NULL; rinfo = rinfo->next) n++;
   if (file == NULL) return;
   sizeb = sizeof(rinfos) - 2 * sizeof(first->prev);
   sized = sizeb / sizeof(double);
   start_swapmat(file,name,n,sized);
   for(rinfo = first; rinfo != NULL;rinfo=rinfo->next)
      fwrite(rinfo,sizeof(double),sized,file);

   save_wf_help(file,name);
}



void write_wf_labels(FILE *file)
{
char s[100];
int i;
char *end;

   for(i=0; i< Num_els(Wf_help) ; i++) {
      strcpy(s,Wf_help[i]);
      end = strchr(s,' ');
      *end = 0;
      fprintf(file,"%15s ",s);
   }
   fprintf(file,"\n");
}

void save_asc_rinfo(FILE *file,rinfos *first,char *name)
{
rinfos *rinfo;

   if (file == NULL) return;
   fprintf(file,"\nbegin_%s\n",name);
   write_wf_labels(file);
   for(rinfo = first;rinfo != NULL;rinfo=rinfo->next) {
      fprintf(file,"%15g ",   rinfo->sangle);
      fprintf(file,"%15g ",   rinfo->sdepth);
      fprintf(file,"%15g ",   rinfo->srange);
      fprintf(file,"%15g ",   rinfo->rangle);
      fprintf(file,"%15g ",   rinfo->rdepth);
      fprintf(file,"%15g ",   rinfo->rrange);
      fprintf(file,"%15g ",   rinfo->time);
      fprintf(file,"%15g ",   rinfo->top_turns);
      fprintf(file,"%15g ",   rinfo->bot_turns);
      fprintf(file,"%15g ",   rinfo->top_bonks);
      fprintf(file,"%15g ",   rinfo->bot_bonks);
      fprintf(file,"%15g ",   rinfo->tot_turns);
      fprintf(file,"%15g ",   rinfo->tot_bonks);
      fprintf(file,"%15g ",   rinfo->utp);
      fprintf(file,"%15g ",   rinfo->ltp);
      fprintf(file,"%15g ",   rinfo->p_z);
      fprintf(file,"%15g ",   rinfo->c_at_r);
      fprintf(file,"%15g ",   rinfo->c_at_s);
      fprintf(file,"%15g ",   rinfo->length);
      fprintf(file,"%15g ",   rinfo->length_c2);
      fprintf(file,"%15g ",   rinfo->geo_intens);
      fprintf(file,"%15g ",   rinfo->bottom_loss);
      fprintf(file,"%15g ",   rinfo->tot_intens);
      fprintf(file,"%15g ",   rinfo->tot_amp);
      fprintf(file,"\n");
   }
   fprintf(file,"end_%s\n",name);
}

void save_rinfo(FILE *mat_file,FILE *asc_file,rinfos *first,char *name)
{
   if (mat_file != NULL) save_mat_rinfo(mat_file,first,name);
   if (asc_file != NULL) save_asc_rinfo(asc_file,first,name);
}


void save_doubles(FILE *file,double *d,int n,char *name)
{

   if (file == NULL) return;
   start_mat(file,name,n,1,0,mat_type(0,0,0));
   fwrite(d,sizeof(double),n,file);
}

void save_bathymetry(FILE *file,btabs *first_btab,char *name)
{
btabs *btab;
long size  = sizeof(struct btabx);
long n;

   if (first_btab == NULL) return;
   if (file == NULL) return;
   for(n=0,btab = first_btab; btab!= NULL; btab = btab->next,n++);
   start_swapmat(file,name,n,size /sizeof(double));
   for(btab = first_btab; btab!= NULL; btab = btab->next) {
      btab->r *= Param.sign_dr;
      /*** zzz change sign of z0, z1,z2 here ***/
      fwrite(btab,size,1,file);
      btab->r *= Param.sign_dr;
   }
}

void save_raw_speeds(FILE *file,environs *environ,params *param)
{
int num_profz = param->num.profz;
int i,num_pnts,num_prof = param->num.profs;
int  num_sect = param->num.sections;
double *temp;
profs *prof;

   if (file == NULL) return;
   Safe_malloc(temp,double,num_profz);

   start_mat(file,"widths",num_profz,2,0,mat_type(0,0,0));
   fwrite(param->profz,sizeof(double),num_profz,file);
   for( i = 0; i < num_profz; i++) {
      *temp = param->smooth.width[i];
      fwrite(temp,sizeof(double),1,file);
   }

   start_mat(file,"raw_c",num_profz,num_prof,0,mat_type(0,0,0));
   for(prof = environ->prof; prof != NULL; prof = prof->next) {
      Memcopy(temp,prof->raw_speed,sizeof(double) * num_profz);
      for(num_pnts = prof->num_pnts; num_pnts <= num_profz; num_pnts++)
         temp[num_pnts-1] = temp[prof->num_pnts-1];
      fwrite(temp,sizeof(double),num_profz,file);
   }

   start_mat(file,"debiased_c",num_profz,num_prof,0,mat_type(0,0,0));
   for(prof = environ->prof; prof != NULL; prof = prof->next) {
      Memcopy(temp,prof->speed,sizeof(double) * num_profz);
      for(num_pnts = prof->num_pnts; num_pnts <= num_profz; num_pnts++)
         temp[num_pnts-1] = temp[prof->num_pnts-1];
      fwrite(temp,sizeof(double),num_profz,file);
   }
   start_swapmat(file,"section_z",num_sect,2);
   for( i = 0; i < num_sect; i++) {
      temp[0] = param->smooth.minz[i];
      temp[1] = param->smooth.sect_width[i];
      fwrite(temp,sizeof(double),2,file);
   }
   free(temp);
}

void save_limits(FILE *file,params *param)
{
   if (file == NULL) return;

   start_mat(file,"limits",2,3,0,mat_type(0,0,0));
   fwrite(&param->top,    sizeof(double),1,file);
   fwrite(&param->bottom, sizeof(double),1,file);	
	fwrite(&Fan.min_range, sizeof(double),1,file);
	fwrite(&Fan.max_range, sizeof(double),1,file);
	fwrite(&param->srange, sizeof(double),1,file);
	fwrite(&param->rrange, sizeof(double),1,file);	
}


void save_speeds(FILE *file,tabtabs *tt,int *idex,rstep_tabs *rs_tab,\
       c_tabs *rctab,params *param,environs *environ)
{
int ns = param->num.sections;

   if (file == NULL) return;
   save_raw_speeds(file,environ,param);
   save_tabtab(file,tt,ns,param->vname.ctab);
   save_index(file,idex,param->num.depths);
   save_rstep(file,rs_tab,ns);
   save_sspr(file,tt,"sspr");
   save_table(file,rctab,ns,"rec_c");
	save_limits(file,param);
}

/****************************************************************************/
/*                   end of outray.c                                        */
/****************************************************************************/

