/****************************************************************************/
/*                 header ray.h                                             */
/****************************************************************************/
/*
 *      Copyright (C) 1992,  Woods Hole Oceanographic Institution
 *    See LICENSE file for use, copying and distribution conditions
 *
 *
 *  jim bowlin feb 1992.
 *
 * Changes: Rich Pawlowicz Aug 93
 *             - changed some of the parameters in params.eigen structure
 */

#ifndef TYPEDEF
#define TYPEDEF(a) typedef struct a a
#endif

TYPEDEF(rinfos);
TYPEDEF(rels);
TYPEDEF(c_tabs);
TYPEDEF(rstep_tabs);
TYPEDEF(params);
TYPEDEF(tabtabs);
TYPEDEF(btabs);
TYPEDEF(pinfos);
TYPEDEF(losses);
TYPEDEF(environs);
TYPEDEF(bathys);
TYPEDEF(profs);
TYPEDEF(units);
TYPEDEF(pindexs);

typedef void   (*derivatives)(rels *s,double *c_inv,double *dtheta);

#define Fan_saves 16

/***
  range;
  depth;
  time;
  angle;
  speed;
  grad;
  length
  length / c^2
  p_z  =  sin(theta)/c
  top_bonks;
  bot_bonks;
  tot_bonks;
  upper_tp;
  lower_tp;
  bottom_loss;
  number;
***/

struct units {
   int    offset;
   double value;
   void   *table;
};

struct params              /* all of the parameters       */
{
   struct {
      units bth_range;
      units bth_depth;
      units ssp_range;
      units ssp_depth;
      units ssp_speed;
      units loss_range;
      units loss_angle;
      units loss_atten;
   } units;
   struct {                /*  step size parameters                  */
      double mult;
      double max;
      double min;
      double cos_factor;    /* used in fast_dr() to adust dl  */
   } step;                  /* according to cos(theta)        */

   struct {
      double tolerance;
      int    iteration;
      int    exact;
   } debias;
   
   struct {
      double diff_miss;  /* how close to caustic to save diffracted arrival */
      int    search_max; /* Test for numerical badness in eigenray search. 
                            (see eigenrays,c) */
      double geo_miss;   /* allowable vertical miss distance for eigenrays  */
      double geo_dangle; /* used to compute geometric amplitude             */

   } eigen;

   double bath_smooth;     /* max width for smoothing bathymetry      */
   double earth_radius;    /* radius of earth in (m)                  */
   int    max_bonks;       /* maximum number of turns allowed         */
   double max_angle;       /* maximum ray angle allowed               */
	double min_cos;         /* cos(max_angle)                          */
   int    bottom_type;     /* 0 = reflecting,  1 = absorbing          */
   struct {                /* controls profile smoothing widths       */
      int levitus;         /* flag: 1 => use built in levitus spacing */
      int auto_max;        /* flag: 1 => get widths from depth grid   */
      dubs *specific;      /* list of specific widths input by user   */
      int  *width;         /* internal table of widths per input z    */
      int  *minz;          /* min depth for each section              */
      int  *sect_width;    /* width of each section                   */
      int  *valid_sect;
      int  num_valid;
   } smooth;

   struct {                /* numbers of things (calculated internally) */
      int sections;
      int profs;
      int depths;
      int profz;
      int tabs;
   } num;

   struct 
   {
      char *init;            /*   the initialization file         */
      char *prof;            /*   the sound speed profiles file   */
      char *bath;            /*   the bathymetry input file       */
      char *loss;            /*   the bottom loss input file      */
      char *mat;             /*   the output MatLab file          */
      char *ascii;           /*   the output ascii file           */
      char *log;             /*   the log file (ascii)            */
      char *cmnd;            /* cmnd line output file             */
   } fname;

   struct {                   /* flags for what to  save          */
      int   initialization;   /*  1 => save,  0 => don't save     */
      int   filenames;
      int   date;
      int   sound_speeds;
      int   bathymetry;
      int   fan;
      int   wavefront;
      int   eigenrays;
      int   standard;
      int   e_only;
      int   everything;
      int   turning_points;
   } saveflg;

   struct  {
      char *ctab;
      char *btab;
      char *rtab;
      char *idex;
      char *wavefront;
      char *fan;
      char *eigenrays;
      char *eigenfront;
      char *init;
   } vname;

   struct {
      int    max_iter;
      double da;
      double min_dz;
		double z_miss;
		int    min_cnt;
   } add_extrema;

   int        margin;       /* number of extra depths in idex    */
   double     z_tol;        /*  (meters) used in hit_root()      */
   double     sign_dr;
   double     sign_z;         
   int        range_depend; /* int for selecting derivitive routine     */
   int        *profz;        /* depths from profile with most # of pnts. */
   int        mprofz;       /* maximum profile depth                    */
   double     source_c;     /* sound speed at source                    */
   double srange;           /* source range                             */
   double rrange;           /* receiver range                           */
   double top;              /* surface of ocean (usually zero)          */
   double bottom;           /* lowest depth (usually 6000)              */
   dubs   *angles;          /*  list of starting angles                 */
	double sdepth;           /* depth of source                          */
	double rdepth;           /* depth of receiver                        */	
   dubs *rdepths;           /* depths of reciever  elements             */
   double angle1;           /* first ray angle                          */
   double angle2;           /* last ray angle                           */
   long   num_angles;       /* number of rays to shoot                  */
   int    saving_pinfo;     /* 1 => save path information               */
};

struct pindexs {
	pindexs *next;
	double   starti;
	double   endi;
};	

struct pinfos {         /* info regarding the writting of path info       */
   double min_range;    /* minimum range for paths                        */
   double max_range;    /* maximum range for paths                        */
   double dr;           /* range  step for paths                          */
   double dt;           /* time   step for paths                          */
   double dl;           /* length step for paths                          */
   long   write_count;  /* total number of path elements written          */
   long   steps_taken;  /* steps taken since last write                   */
   double next_range;   /* next range to write path element (if dr fixed) */
   double next_time;
   double next_length;
   double ray_count;    /* counts number of rays written                  */
   int    selected;     /* set to one if steps_per or fixed_dr are != 0   */
   FILE   *file;
   long   header;
   long   num_cols;
   int    inc_bonks;        /* 1 ==> include points where we bounce       */
   int    saveflg[Fan_saves];  /* flags for what to save along path       */
   int    save_everything;  /* save everything                            */
	double  index_count;      /* counts number of elements written          */
	pindexs *index;          /* keeps track of which ray is where in ...   */
	pindexs *first_index;    /* output.                                    */
};


/************ structures for reading data files ******************/

struct bathys {
   double r;
   double z;
   bathys *next;
};

struct profs {
   double    range;
   long      num_pnts;
   double    *depth;
   double    *speed;
   double    *raw_speed;
   profs     *next;
   profs     *prev;
};

struct losses {
   double    range;
   long      num_pnts;
   double    *angle;
   double    *atten;
   losses     *next;
   losses     *prev;
};

struct environs {

   long     num_bathy;          /* number of bathymetry points           */
   bathys  *bathy;              /* bathymetry structure                  */
   profs   *prof;               /* linked list of sound speed profiles   */
   profs   *last_prof;
   losses  *loss;                /* linked list of bottom loss regions   */
   losses  *last_loss;
};

struct rinfos {          /*  all  start, end info about a single ray  */
double    sangle,        /*  angle of ray at source                   */
          sdepth,        /*  depth of ray at source                   */
          srange,        /*  range of the source                      */
          rangle,        /*  angle of ray at receiver                 */
          rdepth,        /*  depth of ray at receiver                 */
          rrange,        /*  range of receiver                        */
          time,          /*  time of arrival at the receiver          */
          top_turns,     /*  number of top turns                      */
          bot_turns,     /*  number of bottom turns                   */
          top_bonks,     /*  number of top bounces                    */
          bot_bonks,     /*  number of bottom bounces                 */
          tot_turns,     /*  total number of turns                    */
          tot_bonks,     /*  total number of bounces                  */
          utp,           /*  upper turning point at receiver          */
          ltp,           /*  lower turning point at receiver          */
          p_z,           /*  sin(theta) / c(r,z)                      */
          c_at_r,        /*  speed at receiver range                  */
          c_at_s,        /*  speed of sound at source                 */
          length,        /*  closest approach to bottom               */
          length_c2,     /*  range of closest approach to bottom      */
          geo_intens,    /*  geometrical intensity                    */
          bottom_loss,   /*  attenuation due to bottom bouces         */
          tot_intens,    /*  geometric intensity including bottom loss*/
          tot_amp;       /*  total amplitude = ( I c/c0)^1/2          */
rinfos   *prev;          /*  not used                                 */
rinfos   *next;          /*  points to next rinfo                     */
};

struct rels {     /* this is the structure we integrate */
double r,         /* range                    */
       z,         /* depth                    */
       s,         /* sin(theta)               */
       c,         /* cos(theta)               */
       l,         /* path length              */
       t,         /* time                     */
       q,         /* for gausian beams        */
       p,         /* for gausian beams        */		 		 
       l_c2,      /* path length over speed^2 */
       a;
};

struct btabs      /* linked list of tables for calculating bathymetry */
{
   double r;      /* to find z(R) let dr = R - btab.r                 */
   double z0;     /* such that btab.r <= R < btab.next->r             */
   double z1;     /* then z(R) = btab.z0 + dr * ( btab.z1 + dr * btab.z2) */
   double z2;
   double minz;   /* minimum z over this interval                     */
   btabs *next;
};

struct btabx      /* linked list of tables for calculating bathymetry */
{
   double r;      /* to find z(R) let dr = R - btab.r                 */
   double z0;     /* such that btab.r <= R < btab.next->r             */
   double z1;     /* then z(R) = btab.z0 + dr * ( btab.z1 + dr * btab.z2) */
   double z2;
   double minz;   /* minimum z over this interval                     */
};

struct c_tabs {   /* table used for calculating sound speed */
   double z0;
   double c0;     /* c(z) = c0 - (z-z0) * ( c1 + (z-z0) * c2)  */
   double c1;
   double c2;
};

struct rstep_tabs {
   double    z0;
   double   dr0;
   double   dr1;
   double   dr2;
   double   width;
};

struct tabtabs           /* range dependent sound speed structure */
{
   c_tabs *tab1;         /* sound speed table at range r1 */
   c_tabs *tab2;         /* sound speed table at range r2 */
   double r1;            /* range r1                      */
   double r2;            /* range r2                      */
   double r12_inv;       /*  1 /(r2 - r1)                 */
   tabtabs *next;        /* next tabtab or NULL           */
};

/**************** end of data structures ******************************/

void     initialize_ray(int argc, char *argv[]);            /* initray.c */
double   tab_dr(rstep_tabs *rstep,int *idex,double z);      /* ray.c     */
rinfos   *shoot_one_ray(params *param,double angle,long cnt,long num_a);  

rinfos   *shoot(pinfos *pinfo,char *p_name,int path_flag);

/******************* GLOBAL VARIABLES HERE ******************************/

#undef EXTERN

#ifdef Main
      #define EXTERN
#else
      #define EXTERN extern
#endif

EXTERN double     Version;
EXTERN char      *Prog_date;

EXTERN pinfos     Fan,
                  Eigenfan;

EXTERN params     Param;

EXTERN FILE      *Mat_file;
EXTERN FILE      *Prof_file;
EXTERN FILE      *Bath_file;
EXTERN FILE      *Loss_file;
EXTERN FILE      *Asc_file;

/************************ End of Global Variables  ***********************/

/**************************** End of ray.h *******************************/
