/****************************************************************************/
/********************* header reada.h ***************************************/
/****************************************************************************/

/*
 *    Copyright (C) 1993,  Woods Hole Oceanographic Institution
 *    See LICENSE file for use, copying and distribution conditions
 *
 *    Jim Bowlin jbowlin@whoi.edu        May 10, 1993
 *
 *
 */

/*/  Definitions:
*/

#define End_of_file   "[End_of_file]"
#define DELIMITS      "=,/\"{}();$:#"

/*/  Typedefs:
*/

typedef struct iofiles iofiles;
typedef struct commands commands;

/*/  Structures:
*/

struct iofiles {
   FILE    *file;           /* the toke file                 DYN */
   char    *name;           /* the toke file  name           DYN */
   char    *nickname;       /* no path or extension          DYN */
   char    *extension;      /* the extension                 DYN */
   char    *tok;            /* the token to be passed on     DYN */
   char    *string;         /* hold string read in           DYN */
   char    *err_buf;        /* holds error messages          DYN */
   char    *line;           /* line read in from file        DYN */
   int      dynamic;        /* 1 => this ones on the heap        */
   int      max_len;        /* maximum line length               */

   int      start;          /* index of 1st char in token        */
   int      end;            /* index of last char + 1 in token   */
   long     linecnt;        /* number of lines read              */
   int      unget;          /* 1 => repeat last token            */
   int      err_cnt;        /* counts number of errors           */
   int      end_of_file;
  
   /*** the next for elements may be set by the user  ***/
    
   FILE    *log_file;       /* if not NULL echo err msgs here    */
   char    *prog_name;      /* used for reporting errors         */
   int      debug;          /* 1 => print tokens as we go        */
   int      line_echo_off;  /* 1 => show error line              */
};

   
struct commands {
   char      *command;
   int        number;
	int        invoked;
   commands  *next;
};

/*/  Functions:
*/

/************ open and close iofiles ****************************/

void      raw_open_iofile(iofiles *iof,int linelen,char *format);
iofiles  *open_iofile(char *name,int linelen,char *format);
int       close_iofile(iofiles *iof);

/****************** some utilities *****************************/

char    *raw_nickname(char *name);
char    *raw_extension(char *name);
void     default_extension(char **name,char *ext);
void     find_matching(iofiles *iof,char opening,char closing);

/***************** token reading ********************************/

char    *raw_get_token(iofiles *iof);       /*    one token from *iof      */
                                            /* end of line  generates "\n" */

char    *get_token(iofiles *iof);              /* one token from *iof      */
char    *get_a_line(iofiles *iof);             /* read next line           */
char    *get_token_eos(iofiles *iof);          /* ignore ";"               */
int      unget_token(iofiles *iof);            /* just like ungetch        */
void     iof_err(iofiles *iof,char *s);        /* generate error msg       */
void     eof_err(iofiles *iof);                /* end-of-file error msg    */
int      want_tok(iofiles *iof,char *s);       /* compares next token with */
                                               /* *s, reports error if any */
void     skip_past_eos(char *tok,iofiles *iof);
                                               /* looks for next ";"       */

/*************** reading strings and numbers *******************/

long     get_a_long(iofiles *iof);
double   get_a_dub(iofiles *iof);
int      get_an_int(iofiles *iof);
char    *get_a_string(iofiles *iof);

/*************** replacable parameters **************************/

void     add_command(char *s);           /* define parameter $n++$       */
int      check_commands(void);           /* makes sure all commands have */
                                         /* been used.                   */
/****************************************************************************/
/******************** end of header reada.h *********************************/
/****************************************************************************/

