/****************************************************************************/
/*                 program utils.c                                            */
/****************************************************************************/
/*
 *      Copyright (C) 1992,  Woods Hole Oceanographic Institution
 *    See LICENSE file for use, copying and distribution conditions
 *
 *
 *  jim bowlin feb 1992.
 *
 *  General purpose utilities.  See utils.h for definitions
 */

#define Utils

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "utils.h"

#undef Utils


/***************** static Global variables *****************************/

static char    More_str[] = "                    ----- More -----";
static long    Max_lines = 23;

/*************** rewrite of some std functions **************************/
/*
 * Note that all of these versions are capitalized.
 */

char *Strlwr(char *s)
{
int len,i;

    len = strlen(s);
    for(i = 0; i < len; i++) s[i] = tolower(s[i]);
    return(s);
}

char *Strdup(char *s)
{
int i;
char *out;

   i = strlen(s);
   out = (char*)  calloc(1,i+1);
   if (out != NULL) strcpy(out,s);
   return(out);
}

char *Strcpycat(char *s1,char *s2)
{
char *dest;

   Safe_calloc(dest,char,strlen(s1) + strlen(s2) + 1);
   strcpy(dest,s1);
   strcat(dest,s2);
   return(dest);
}

double Strtod(char *s, char **sret)
{
  double r; /* result */
  int e; /* exponent */
  double d; /* scale */
  int sign; /* +- 1.0 */
  int esign;
  int i;
  int flags=0;

  r = 0.0;
  sign = 1;
  e = 0;
  esign = 1;

  while ((*s == ' ') || (*s == '\t'))
     s++;
  if (*s == '+')
     s++;
  else if (*s == '-')
  {
     sign = -1;
     s++;
  }

  while ((*s >= '0') && (*s <= '9'))
  {
    flags |= 1;
    r *= 10.0;
    r += *s - '0';
    s++;
  }

  if (*s == '.')
  {
    d = 0.1;
    s++;
    while ((*s >= '0') && (*s <= '9'))
    {
      flags |= 2;
      r += d * (*s - '0');
      s++;
      d /= 10.0;
    }
  }

  if (flags == 0)
  {
    if (sret) *sret = s;
    return 0;
  }

  if ((*s == 'e') || (*s == 'E'))
  {
    s++;
    if (*s == '+')
      s++;
    else if (*s == '-')
    {
      s++;
      esign = -1;
    }
    if ((*s < '0') || (*s > '9'))
    {
      if (sret) *sret = s;
      return r;
    }

    while ((*s >= '0') && (*s <= '9'))
    {
      e *= 10;
      e += *s - '0';
      s++;
    }
  }

  if (esign < 0)
    for (i = 1; i <= e; i++)
      r /= 10.0;
  else
    for (i = 1; i <= e; i++)
      r *= 10.0;

  if (sret) *sret = s;
  return r * sign;
}

/******* these functions handle some standard error reporting *****/

void lputs(char *s)
{
   fputs(s,stdout);
   if (Log_file == NULL) return;
   fputs(s,Log_file);
}

void vlputs(char *s)
{
   if (Verbose)
		fputs(s,stdout);
   if (Log_file == NULL)
		return;
   fputs(s,Log_file);
}

void fatal(char *s)
{
char temp[80];

   if (s != NULL) {
      lputs(s);
      lputs("\n");
   }
   if (Prog_name != NULL)
      sprintf(temp,"Fatal error in %s.\n",Prog_name);
   else
      sprintf(temp,"Fatal error.\n");

   lputs(temp);
   exit(-1);
}

void warning(char *s)
{
char temp[80];

   sprintf(temp,"%s: warning %s.\n",Prog_name,s);
   lputs(temp);
}


void vprint(char *s)
{
   if (!Verbose) return;
   printf("%s",s);
   fflush(stdout);
}

void ldbug(char *s)
/*
 *  send message to log file as well as to dbug();
 */
{

   if (Debug) {
     lputs(s);
     lputs("\n");
     return;
   }
   if (Log_file == NULL) return;
   fputs(s,Log_file);
   fputs("\n",Log_file);
}

void mputs(char *s)
{
static long linecnt;
int    i;

   if ((!Screen) || (++linecnt < Max_lines)) {
      puts(s);
      return;
   }
   linecnt = 0;
   for ( i = 0; i < Num_els(More_str); i ++) putchar(More_str[i]);
   fflush(stdout);
   getchar();
   putchar('\r');
   for ( i = 0; i < Num_els(More_str); i ++) putchar(' ');
   putchar('\r');
   puts(s);
}

char *s2dhms(unsigned int n)
{
char  tmp[80];
unsigned int secs,mins,hours,days;

    secs = n % 60;
    if ((n /= 60) == 0) {
       sprintf(tmp,"%d secs",secs);
       return( strdup(tmp));
    }

    mins = n % 60;
    if ((n /= 60) == 0) {
       sprintf(tmp,"%d mins %d secs",mins,secs);
       return( strdup(tmp));
    }
    hours = n % 24;
    if ((n /= 24) == 0) {
       sprintf(tmp,"%d hours %d mins %d secs",hours,mins,secs);
       return( strdup(tmp));
    }
    days = n;
    sprintf(tmp,"%d days %d hours %d mins %d secs",days,hours,mins,secs);
    return( strdup(tmp));
}

/****************************************************************************/
/*                End of utils.c                                            */
/****************************************************************************/


