/****************************************************************************/
/*********************** header utils.h *************************************/
/****************************************************************************/
/*
 *    Copyright (C) 1993,  Woods Hole Oceanographic Institution
 *    See LICENSE file for use, copying and distribution conditions
 *
 *    Jim Bowlin jbowlin@whoi.edu        May 10, 1993
 */

/*/  Definitions:
*/

#define Safe_calloc(var,type,num) \
if ((var = (type*) calloc((num),sizeof(type) )) ==NULL) fatal("Out of Memory")

#define Safe_malloc(var,type,num) \
if ((var = (type*) malloc(sizeof(type) * num)) ==NULL) fatal("Out of Memory")

#define Num_els(x) (long) ( sizeof(x) / sizeof(x[0]) )
#define Min(a,b) ( ( (a) < (b) ) ? (a) : (b) )
#define Max(a,b) ( ( (a) > (b) ) ? (a) : (b) )
#define Sign(a,b) ( (b) > 0.0 ? fabs(a) : -fabs(a) )
#define Memcopy(a,b,c)  memcpy( (char *)(a),(char *) (b),(c))
#define SEEK_SET  0
#define SEEK_CUR  1
#define SEEK_END  2
#define Pi    3.14159265358979


#define Set_Prog_name(x)  Prog_name = Strdup(x)
#define Set_Debug(x)      Debug     = (x)
#define Set_Verbose(v)    Verbose   = (x)
#define Set_Screen(s)     Screen    = (x)
#define Set_Log_file(f)   Log_file  = (x)

#define SHIFT3(a,b,c)  (a) = (b); (b) = (c)

#define SHIFT4(a,b,c,d)  (a) = (b); (b) = (c); (c) = (d)


/*  Typedefs:
*/

typedef struct dubs dubs;

/*/  Structures:
*/

struct dubs {
   double d;
   dubs   *next;
};

/*/  Functions:
*/

char    *Strdup(char *s);
char    *Strlwr(char *s);
char    *Strcpycat(char *s1,char *s2);
double   Strtod(char *s,char **end);

void    fatal(char*s);
void    ldbug(char *s);
void    lputs(char *s);
void    vlputs(char *s);
void    vprint(char *s);
void    warning(char *s);
void    mputs(char *s);
char   *s2dhms(unsigned int n);


/*/  Gobal variables:
*/

#undef EXTERN

#ifdef Utils
   #define EXTERN
   #define EQ(x)  x
#else
   #define EXTERN extern
   #define EQ(x)

#endif

EXTERN char  *Prog_name EQ( = "utils.c");
EXTERN int   Debug;
EXTERN int   Verbose;
EXTERN int   Screen;
EXTERN FILE  *Log_file;

/****************************************************************************/
/******************* end of  header utils.h *********************************/
/****************************************************************************/
